#include "ui.h"
#include "SDL_events.h"
#include "aliases/aliases.h"
#include "window.h"
#include <stdio.h>

typedef struct ui_elem_colours ui_elem_colours_t;
struct ui_elem_colours {
  colour_t fill;
  colour_t border;
};

INTERNAL ui_elem_colours_t colours[COUNT_UI_ELEM] = {
    [UI_ELEM_NODE] =
        (ui_elem_colours_t){
            .fill = (colour_t){.abgr = 0xff2c84b7},
            .border = (colour_t){.abgr = 0xff315c89},
        },
    [UI_ELEM_BUTTON] =
        (ui_elem_colours_t){
            .fill = (colour_t){.abgr = 0xff89a83c},
            .border = (colour_t){.abgr = 0xff768432},
        },
};

bool aabb(const ui_elem_t *elem, i32 x, i32 y) {
  return x > elem->rect.topleft.x && x <= elem->rect.topleft.x + elem->rect.w &&
         y > elem->rect.topleft.y && y <= elem->rect.topleft.y + elem->rect.h;
}

void reset_ui_ctx(ui_ctx_t *ctx) {
  ctx->count = 0;
  ctx->mouse_down = false;
  ctx->mouse_up = false;
  ctx->rel_x = 0;
  ctx->rel_y = 0;
}

void handle_ui_events(const window_t *wnd, ui_ctx_t *ctx,
                      const SDL_Event *event) {
  switch (event->type) {
  case SDL_MOUSEMOTION:
    ctx->rel_x = event->motion.xrel;
    ctx->rel_y = event->motion.yrel;

    if (wnd->id == event->motion.windowID) {
      ctx->mouse_x = event->motion.x;
      ctx->mouse_y = event->motion.y;
      ctx->wnd = wnd;

      break;
    }
  case SDL_MOUSEBUTTONDOWN:
    if (wnd->id == event->button.windowID) {
      ctx->mouse_x = event->button.x;
      ctx->mouse_y = event->button.y;
      ctx->mouse_down = true;
      ctx->wnd = wnd;
    }

    break;
  case SDL_MOUSEBUTTONUP:
    if (wnd->id == event->button.windowID) {
      ctx->mouse_x = event->button.x;
      ctx->mouse_y = event->button.y;
      ctx->mouse_up = true;
      ctx->wnd = wnd;
    }

    break;
  }
}

bool button(const window_t *wnd, ui_ctx_t *ctx, rect_t rect) {
  if (ctx->count + 1 >= MAX_UI_ELEMENTS) {
    return false;
  }

  ui_elem_t elem = (ui_elem_t){
      .id = (ctx->count)++,
      .rect = rect,
      .type = UI_ELEM_BUTTON,
  };

  fill_rect(wnd, rect, colours[UI_ELEM_BUTTON].fill);
  draw_rect(wnd, rect, colours[UI_ELEM_BUTTON].border);

  if (wnd != ctx->wnd) {
    return false;
  }

  if (!aabb(&elem, ctx->mouse_x, ctx->mouse_y)) {
    return false;
  }

  ctx->hovered = elem.id;

  if (ctx->mouse_down) {
    ctx->active = elem.id;
  }

  if (ctx->mouse_up && ctx->hovered == elem.id && ctx->active == elem.id) {
    ctx->hovered = ctx->active = -1;
    return true;
  }

  return false;
}

rect_t node(const window_t *wnd, ui_ctx_t *ctx, rect_t rect) {
  if (ctx->count + 1 >= MAX_UI_ELEMENTS) {
    return (rect_t){0};
  }

  ui_elem_t elem = (ui_elem_t){
      .id = (ctx->count)++,
      .rect = rect,
      .type = UI_ELEM_BUTTON,
  };

  fill_rect(wnd, rect, colours[UI_ELEM_BUTTON].fill);
  draw_rect(wnd, rect, colours[UI_ELEM_BUTTON].border);

  if (wnd != ctx->wnd) {
    return rect;
  }

  if (ctx->mouse_up) {
    ctx->hovered = ctx->active = -1;
  }

  if (ctx->hovered == elem.id && ctx->active == elem.id) {
    return (rect_t){
        .topleft.x = rect.topleft.x + ctx->rel_x,
        .topleft.y = rect.topleft.y + ctx->rel_y,
        .w = rect.w,
        .h = rect.h,
    };
  }

  if (!aabb(&elem, ctx->mouse_x, ctx->mouse_y)) {
    return rect;
  }

  ctx->hovered = elem.id;

  if (ctx->mouse_down) {
    ctx->active = elem.id;
  }

  return rect;
}
